/**************************************************************************
***    
*** Copyright (c) 1995-2000 Regents of the University of California,
***               Andrew E. Caldwell, Andrew B. Kahng and Igor L. Markov
***
***  Contact author(s): abk@cs.ucla.edu, imarkov@cs.ucla.edu
***  Affiliation:   UCLA, Computer Science Department,
***                 Los Angeles, CA 90095-1596 USA
***
***  Permission is hereby granted, free of charge, to any person obtaining 
***  a copy of this software and associated documentation files (the
***  "Software"), to deal in the Software without restriction, including
***  without limitation 
***  the rights to use, copy, modify, merge, publish, distribute, sublicense, 
***  and/or sell copies of the Software, and to permit persons to whom the 
***  Software is furnished to do so, subject to the following conditions:
***
***  The above copyright notice and this permission notice shall be included
***  in all copies or substantial portions of the Software.
***
*** THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
*** EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
*** OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
*** IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
*** CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
*** OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
*** THE USE OR OTHER DEALINGS IN THE SOFTWARE.
***
***
***************************************************************************/




//created by Andrew Caldwell and Igor Markov on 09/19/98

#ifndef __REGULAR_GAIN_BUCKET_H__
#define __REGULAR_GAIN_BUCKET_H__

#include "ABKCommon/abkcommon.h"
#include <vector>
using std::vector;
#include <algorithm>
using std::min;
using std::max;
#include "baseGainBucket.h"

class RegularBucketArray;
typedef RegularBucketArray BucketArray;

class RegularBucketArray : public BaseBucketArray
{
  public:
    RegularBucketArray() : BaseBucketArray() {}
    ~RegularBucketArray(){}

    void setMaxGain(unsigned maxG);
    void push(GenericGainElement& elmt);
};

//____________________IMPLEMENTATIONS__________________________//


inline void RegularBucketArray::setMaxGain(unsigned maxG)
{ 	
    _gainOffset = maxG;
    _maxBucketIdx =  2 * _gainOffset;
    this->insert(end(), _maxBucketIdx+1-size(), GainList());
    _highestNonEmpty = _highestValid = 0;
}

inline void RegularBucketArray::push(GenericGainElement& elmt)
{
    abkassert(_maxBucketIdx != -1, 
	"must call setMaxGain before using the RegularBucketArray");

    int bucket = max(min(_maxBucketIdx,  elmt._gain+_gainOffset), 0);
    (*this)[bucket].push(elmt);
    _highestNonEmpty = max(_highestNonEmpty, bucket);
}

#endif

