/**************************************************************************
***
***    Copyright (c) 1995-2000 by ABKGroup, UCLA VLSI CAD Laboratory,
***    UCLA Computer Science Department, Los Angeles, CA 90095-1596 USA,
***    and by the Regents of the University of California.
***    All rights reserved.
***
***    No portion of this file may be used, copied, or transmitted for
***    any purpose, including any research or commercial purpose,
***    without the express written consent of Professor Andrew B. Kahng,
***    UCLA Computer Science Department, 3713 Boelter Hall, Los Angeles,
***    CA 90095-1596 USA.   Any copy made of this file must include this
***    notice.    ABKGroup and the Regents of the University of California
***    retain title to all copies of this file and/or its compiled
***    derivatives.
***
***************************************************************************/

#include <fstream.h>
#include "ABKCommon/abkcommon.h"
#include "DB/DB.h"

// Dump net and pin names alphabetically
class CompareAlphabetically
{
   public:
    CompareAlphabetically(){}

    bool operator() ( const char* c1, const char* c2) const
    {
        int cmpResult = strcasecmp(c1, c2);
        if(cmpResult < 0) return true;
        else if (cmpResult > 0) return false;
        else
        {
            int caseSens=strcmp(c1,c2);
            if (caseSens<0) return true;
            else if (caseSens>0) return false;
            else return (c1<c2);
        }
        return false; //never happen
    }
};

int main(int argc, char *argv[])
{
  StringParam auxFileName("f",argc,argv);
  StringParam outFileName("o",argc,argv);
  abkfatal(auxFileName.found()&&outFileName.found(),
      "Usage: prog -f filename.aux -o outfile");
  DB db(auxFileName);
  const dbNetlist &netlist=db.getNetlist();
  const unsigned numNets=netlist.getNumNets();
  vector<char*> netNames(numNets,static_cast<char*>(NULL));
  {
  for (unsigned i=0;i<numNets;i++)
  {
      const char* name=netlist.getNetByIdx(i).getName();
      netNames[i]=new char[strlen(name)+1];
      strcpy(netNames[i],name);
  }
  }

  CompareAlphabetically c;
  sort(netNames.begin(),netNames.end(),c);

  const unsigned numPins=netlist.getNumPins();
  vector<char*> pinNames(numPins,static_cast<char*>(NULL));
  {
  for (unsigned i=0;i<numPins;i++)
  {
      const dbPin &pin=netlist.getPinByIdx(i);
      const char *pinname=pin.getMaster().getName();
      const char *cellname=pin.getCell().getName();
      pinNames[i]=new char[strlen(cellname)+strlen(pinname)+2];
      strcpy(pinNames[i],cellname);
      strcat(pinNames[i]," ");
      strcat(pinNames[i],pinname);
  }
  }

  sort(pinNames.begin(),pinNames.end(),c);
  ofstream os(outFileName);
  os << "NET NAMES" << endl << endl << endl;
  {
  for (unsigned i=0;i<numNets;i++)
      os <<"\t" << netNames[i] << endl;
  }
  os << endl << endl << endl;
  os << "PIN NAMES" << endl << endl << endl;
  {
  for (unsigned i=0;i<numPins;i++)
      os <<"\t" << pinNames[i] << endl;
  }



  {
  for (unsigned i=0;i<numNets;i++)
      delete netNames[i];
  }
  {
  for (unsigned i=0;i<numPins;i++)
      delete pinNames[i];
  }

  return 0;
}

