/**************************************************************************
***
***    Copyright (c) 1995-1998 by ABKGroup, UCLA VLSI CAD Laboratory,
***    UCLA Computer Science Department, Los Angeles, CA 90095-1596 USA,
***    and by the Regents of the University of California.
***    All rights reserved.
***
***    No portion of this package may be used, copied, or transmitted for
***    any purpose, including any research or commercial purpose,
***    without the express written consent of Professor Andrew B. Kahng,
***    UCLA Computer Science Department, 3713 Boelter Hall, Los Angeles,
***    CA 90095-1596 USA.   Any copy made of this file must include this
***    notice.    ABKGroup and the Regents of the University of California
***    retain title to all copies of this file and/or its compiled
***    derivatives.
***
***************************************************************************/

// Created by Igor Markov

#include "partProb.h"
#include "partitionIds.h"
#include "allTo1Gen.h"

unsigned AllToOneGen::counter=0;

vector<double> AllToOneGen::generateSoln(Partitioning& curPart)
{
   unsigned            numPart  = _problem.getNumPartitions();
   const HGraphFixed&  hgraph   = _problem.getHGraph();
   unsigned            numNodes = _problem.getHGraph().getNumNodes(); 
   const Partitioning& fixed    = _problem.getFixedConstr(); 
   vector<double>      balances(numPart);

   for(unsigned node=0; node!=numNodes; node++)
   {
     unsigned orig=counter % numPart;
     unsigned nodeGoesTo=orig;
     while ( ! fixed[node].isInPart(nodeGoesTo) )
     {
        nodeGoesTo= (nodeGoesTo + 1) % numPart;
        abkfatal3(nodeGoesTo!=orig," Node ",node,
                  " can not go into any partition\n");
     }
     
     curPart[node].setToPart(nodeGoesTo);
     balances[nodeGoesTo]+=hgraph.getNodeByIdx(node).getWeight();
   }
   counter++;
   return balances;
};

