/**************************************************************************
***
***    Copyright (c) 1995-1997 by ABKGroup, UCLA VLSI CAD Laboratory,
***    UCLA Computer Science Department, Los Angeles, CA 90095-1596 USA,
***    and by the Regents of the University of California.
***    All rights reserved.
***
***    No portion of this file may be used, copied, or transmitted for
***    any purpose, including any research or commercial purpose,
***    without the express written consent of Professor Andrew B. Kahng,
***    UCLA Computer Science Department, 3713 Boelter Hall, Los Angeles,
***    CA 90095-1596 USA.   Any copy made of this file must include this
***    notice.    ABKGroup and the Regents of the University of California
***    retain title to all copies of this file and/or its compiled
***    derivatives.
***
***************************************************************************/

#include "trivStats.h"
#include "partitioning.h"
#include "subPartProb.h"
#include "DB.h"
#include "dbHGr.h"

main(int argc, const char *argv[]) 
{
    StringParam  auxFileName("f",argc,argv);  
    StringParam  plFileName("pl",argc,argv);
    abkfatal(auxFileName.found()," -f auxFileName not found");
    DB db(auxFileName);

    Placement pl(0);
    if (plFileName.found())
    {
        ifstream plFile(plFileName);
        Placement plIn(plFile);
        plFile.close();
        pl = plIn;
    }
    else
        pl = db.spatial.locations;

    HGraphFixed hg(HGfromDB(db.getNetlist()));

    //BBox chipBox;
    //for(unsigned i =hg.getNumTerminals(); i<pl.getSize(); i++)
    //    chipBox += pl[i];

    LayoutBBoxes boxes(2); //, wholeChip(1);
    boxes[0] = BBox(0,0,100000,100000);
    boxes[1] = BBox(0,100000,100000,150000);
    //wholeChip[0] = chipBox;

    hg.saveAsNetDAre("t8orig.netD","t8orig.are");
    ofstream plout("t8orig.pl");
    plout << pl;
    plout.close();
    //cerr<<"Replicate the original HGraph.....";
    //SubPartitioningProblem replicateProb(wholeChip, hg, pl);
    //replicateProb.save("t8repl");

    cerr<<"Generating partProb.....";
    SubPartitioningProblem problem(boxes, hg, pl);

    problem.save("t8");

    cout<<"Stats for Problem"<<endl;
    const HGraphFixed &hgraph = problem.getHGraph();

    vector<unsigned> nodeDegs;
    double           totalArea = 0;
    double   	     maxArea = 0;
    vector<double>   nodeAreas;

    for(itHGFNodeGlobal n = hgraph.nodesBegin(); n != hgraph.nodesEnd(); n++)
    {
	nodeDegs.push_back((*n)->getDegree());
	totalArea += (*n)->getWeight();
	nodeAreas.push_back((*n)->getWeight());	
	maxArea = max(maxArea, (*n)->getWeight());
    }

    vector<unsigned> netDegs;
    for(itHGFEdgeGlobal e = hgraph.edgesBegin(); e != hgraph.edgesEnd(); e++)
    {
	netDegs.push_back((*e)->getDegree());
    }

    cout<<"___Node Stats______"<<endl<<endl << flush;
    cout<<"Total Nodes: "<<hgraph.getNumNodes()<<endl;
    cout<<"  Terminals: "<<hgraph.getNumTerminals()<<endl;
    cout<<"  Non-Terms: "<<hgraph.getNumNodes()-hgraph.getNumTerminals()<<endl;
    cout<<"TotalNodeArea: "<<totalArea<<endl;
    cout<<" Node Areas: "<<TrivialStats(nodeAreas)<<endl;
    cout<<" Largest Node is "<<(maxArea / totalArea)*100<<
			"% of total area"<<endl;

    cout<<"Node Degrees: "<<TrivialStats(nodeDegs)<<endl;

    cout<<endl<<"___Net Stats______"<<endl;

    cout<<"Total Nets:  "<<hgraph.getNumEdges()<<endl;
    cout<<" Net Degrees: "<<TrivialStats(netDegs)<<endl;

    return 0;
}
	
