/**************************************************************************
***    
*** Copyright (c) 1995-2000 Regents of the University of California,
***               Andrew E. Caldwell, Andrew B. Kahng and Igor L. Markov
***
***  Contact author(s): abk@cs.ucla.edu, imarkov@cs.ucla.edu
***  Affiliation:   UCLA, Computer Science Department,
***                 Los Angeles, CA 90095-1596 USA
***
***  Permission is hereby granted, free of charge, to any person obtaining 
***  a copy of this software and associated documentation files (the
***  "Software"), to deal in the Software without restriction, including
***  without limitation 
***  the rights to use, copy, modify, merge, publish, distribute, sublicense, 
***  and/or sell copies of the Software, and to permit persons to whom the 
***  Software is furnished to do so, subject to the following conditions:
***
***  The above copyright notice and this permission notice shall be included
***  in all copies or substantial portions of the Software.
***
*** THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
*** EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
*** OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
*** IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
*** CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
*** OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
*** THE USE OR OTHER DEALINGS IN THE SOFTWARE.
***
***
***************************************************************************/







#include "ABKCommon/abkcommon.h"
#include "RBPlacement.h"

int main(int argc, const char *argv[])
{
  NoParams         noParams       (argc,argv);
  BoolParam        helpRequest    ("help",argc,argv);
  BoolParam        helpRequest1   ("h",   argc,argv);

  RBPlacement::Parameters rbParams(argc,argv);
  StringParam   saveAsNodes("saveAsNodes", argc, argv);
  StringParam   plotNets("plotNets", argc, argv);
  StringParam   plotNodes("plotNodes", argc, argv);
  StringParam   plotNodesAndNets("plot", argc, argv);
  StringParam   plotNodesWNames("plotNodesWNames", argc, argv);
  StringParam   plotNodesAndNetsWNames("plotWNames", argc, argv);

  if (noParams.found() || helpRequest.found() || helpRequest1.found())
  {
     cout << "  Use '-help' or '-f filename.aux' " << endl;
     cout << "  Other options: \n"
          << "   -saveAsNodes     base_filename\n"
          << "   -plotNets        base_fileName\n"
          << "   -plotNodes       base_fileName\n"
          << "   -plotWNodesNames base_fileName\n"
          << "   -plot            base_fileName   (plots nodes and nets)\n"
          << "   -plotWNames      base_fileName   (the above + node names)\n"
          << endl;
     return 0;
  }
  cout<<"RBPlacement Parameters"<<endl;
  cout<<rbParams<<endl;


  StringParam auxFile("f", argc, argv);
  abkfatal(auxFile.found(), "must have -f <auxfilename>");
  RBPlacement rbplace(auxFile, rbParams);


// add code that produces gnuplot files by traversing the hypergraph
// and the layout from rbplace   

  if(saveAsNodes.found())
	 rbplace.saveAsNodesNetsWts(saveAsNodes);

  if(plotNets.found())
    	 rbplace.saveAsPlot(argc ,argv ,plotNets);

  if(plotNodes.found())
    	 rbplace.saveAsPlot(argc ,argv ,plotNodes);

   if(plotNodesAndNets.found())
	 rbplace.saveAsPlot(argc ,argv ,plotNodesAndNets);

  if(plotNodesWNames.found())
    	 rbplace.saveAsPlot(argc ,argv ,plotNodesWNames);

   if(plotNodesAndNetsWNames.found())
	 rbplace.saveAsPlot(argc ,argv ,plotNodesAndNetsWNames);
 
  return 0;
}



