/**************************************************************************
***
***    Copyright (c) 1995-1997 by ABKGroup, UCLA VLSI CAD Laboratory,
***    UCLA Computer Science Department, Los Angeles, CA 90095-1596 USA,
***    and by the Regents of the University of California.
***    All rights reserved.
***
***    No portion of this file may be used, copied, or transmitted for
***    any purpose, including any research or commercial purpose,
***    without the express written consent of Professor Andrew B. Kahng,
***    UCLA Computer Science Department, 3713 Boelter Hall, Los Angeles,
***    CA 90095-1596 USA.   Any copy made of this file must include this
***    notice.    ABKGroup and the Regents of the University of California
***    retain title to all copies of this file and/or its compiled
***    derivatives.
***
***************************************************************************/

// Created: May 10, 1999 by Andrew Caldwell,
//	taken from capoPlacer.cxx by Igor Markov and Andrew Caldwell


//CHANGES

#include "capoPlacer.h"
#include "capoBlock.h"
#include "RBPlacement.h"
#include "placeEvals.h"
#include "trivStats.h"
#include "xyDraw.h"
#include "RBPlacement.h"


bool CapoPlacer::doOneLayer(AllowedPartDir partDir)
{
    Timer capoLoopTimer;

    vector<CapoBlock*>& activeLayout = _layout[_layerNum %2];
    vector<CapoBlock*>& newLayout    = _layout[(_layerNum +1) %2];
    _curLayout = &(_layout[_layerNum %2]);

    newLayout.clear();

    //refine each block in the active layout, and put the
    //newly created blocks into newLayout

    _totalOverfill = 0;

//NOTE:  This has been put back to do alternating RB
/*
    if(partDir == VOnly)
   	doVOnlyLayer(activeLayout, newLayout, _layerNum);
    else if(partDir == HOnly) 
   	doHOnlyLayer(activeLayout, newLayout, _layerNum);
    else
*/
   	doHAndVLayer(activeLayout, newLayout, _layerNum);

    _curLayout = &(_layout[(_layerNum+1) %2]);	//switch to the new layer

    if(_params.verb.forMajStats > 0) cerr<<endl;

    capoLoopTimer.stop();

    if(true || _params.verb.forMajStats > 1)
    {
	  cout<<endl;
          cout<<"Stats for Layer "<<_layerNum<<endl;

	  BBoxPlEval		hpWLEval   (_hgraphOfNetlist, _placement);
	  BBoxWChengPlEval      chengWLEval(_hgraphOfNetlist, _placement);
	  BBoxWRSMTPlEval       rsmtWLEval (_hgraphOfNetlist, _placement);

	  double		bboxWL  = hpWLEval.getCost();
	  double		chengWL = chengWLEval.getCost();
	  double		rsmtWL  = rsmtWLEval.getCost();

	  cout<<"Layer "<<_layerNum<<" Stats:  WL   (HP): "<<bboxWL <<"\n";
	  cout<<"Layer "<<_layerNum<<" Stats: WWL(Cheng): "<<chengWL<<"\n";
	  cout<<"Layer "<<_layerNum<<" Stats: WWL (RSMT): "<<rsmtWL <<"\n";

	  cout<<"Layer "<<_layerNum<<" Stats:  Time:  " 
	      << capoLoopTimer.getUserTime() << endl;

	  cout<<"Layer "<<_layerNum<<" Stats:  Memory: "<<MemUsage()<<endl;

	  _totalLayoutArea = _totalOverfill = _maxOverfill = 0;

	  for(unsigned b = 0; b < _curLayout->size(); b++)
	  {
		double cap = (*_curLayout)[b]->getCapacity();
		double ovr = (*_curLayout)[b]->getOverfill();
		_totalLayoutArea += cap;
		_totalOverfill   += ovr;

		_maxOverfill = max(_maxOverfill, ovr/cap);
	  }
	  for(b = 0; b < _placedBlocks.size(); b++)
	  {
		double cap = _placedBlocks[b]->getCapacity();
		double ovr = _placedBlocks[b]->getOverfill();
		_totalLayoutArea += cap;
		_totalOverfill   += ovr;

		_maxOverfill = max(_maxOverfill, ovr/cap);
	  }

	  cout<<"Layer "<<_layerNum<<" Ave Overfill: "
			<<100*(_totalOverfill/_totalLayoutArea)<<"%"<<endl;
	  cout<<"Layer "<<_layerNum<<" Max Overfill: "
			<<_maxOverfill*100<<"%"<<endl;
	
	
	  cout<<endl;
    }

    if( _params.doOverlapping &&
        (_layerNum >= _params.startOverlappingLayer) &&
        (_layerNum <= _params.endOverlappingLayer))
    {
          capoLoopTimer.start();
          doOverlapping(*_curLayout);
          capoLoopTimer.stop();

          if(true || _params.verb.forMajStats > 1)
          {
	    double hpWL = estimateWL(); 
	    cout<<"Layer "<<_layerNum<<" Stats:  Post-Overlapping HP WL: "
					<<hpWL<<"\n";
	    cout<<"Layer "<<_layerNum<<" Stats:  Overlapping Time:  " 
	        << capoLoopTimer.getUserTime() << endl;
	    cout<<endl;

	    _totalOverfill = 0;
	    for(unsigned i = 0; i < _curLayout->size(); i++)
		_totalOverfill += (*_curLayout)[i]->getOverfill();
	    cout<<"Layer "<<_layerNum<<" Stats:  Ave Relative Overfill: "
		<<_totalOverfill / _totalLayoutArea<<endl;
          }
    }
 
    if (_params.plotBlocks) plotBlocks(_layerNum,*_curLayout);

    if(_params.saveLayerBBoxes)
    {
	LayoutBBoxes regions;
   	for(unsigned box = 0; box < _curLayout->size(); box++)
            regions.push_back((*_curLayout)[box]->getBBox());

	char* layerStr = new char[30];
        sprintf(layerStr, "LevelBBoxes_%d", _layerNum);
        regions.save(layerStr);
    }

    _layerNum++;

    return _curLayout->size() != 0;
}


void CapoPlacer::doVOnlyLayer(vector<CapoBlock*>& curLayout,
			      vector<CapoBlock*>& newLayout,
			      unsigned layerNum)
{
    if(_params.verb.forMajStats > 0)
    {
        cerr<<"Layer{V}: "<<layerNum<<"  Blocks: "<<curLayout.size()<<"\n";
        cout<<"Layer{V}: "<<layerNum<<"  Blocks: "<<curLayout.size()<<"\n";
    }

    unsigned totalNumBlks = _curLayout->size() + _placedBlocks.size();

    abkfatal(totalNumBlks > 0, 
		"There are no blocks at the start of doVOnlyLayer");

    unsigned minBlkSize = _hgraphOfNetlist.getNumNodes()/totalNumBlks;
    minBlkSize = minBlkSize < 50 ? 0 : (minBlkSize*2)/3;
	//min becomes 0 when we get to small block partitioning
	//so that we can't get stuck, unable to partition a very small
	//multi-row block

    bool didVSplit = true;

    newLayout.clear();

    while(didVSplit)
    {
	didVSplit = false;
	if(_params.verb.forMajStats > 1)
	    cout<<"Begin V-Split pass"<<endl;

        for(unsigned b = 0; b < curLayout.size(); b++)
	    curLayout[b]->_canSplitBlock = true;

	cerr<<"   [";

        for(b = 0; b < curLayout.size(); b++)
        {
	    CapoBlock& block = *curLayout[b];

	    if(_params.verb.forMajStats > 3)
	    {
	       cout<<"Block: "<<b<<" ["<<block.getNumCells() <<" cells]"<<endl;
	       if(_params.verb.forMajStats > 4)
	           cout<<block<<endl;
	    }

	    bool blockWasSplit = refineBlock(block,newLayout,_layerNum, 
						VOnly, minBlkSize);
	    didVSplit |= blockWasSplit;

	    if(blockWasSplit)
	        delete curLayout[b];

            curLayout[b]=0;
	}

	cerr<<"]\n";
	if(didVSplit)	//prepare to do it again...
	{
            curLayout = newLayout;
            newLayout.clear();
	}
    }
    curLayout.clear();	//everything is in newLayout
}

void CapoPlacer::doHOnlyLayer(vector<CapoBlock*>& curLayout,
			      vector<CapoBlock*>& newLayout,
			      unsigned layerNum)
{
    if(_params.verb.forMajStats > 0)
    {
        cerr<<"Layer{H}: "<<layerNum<<"  Blocks: "<<curLayout.size()<<"\n";
        cout<<"Layer{H}: "<<layerNum<<"  Blocks: "<<curLayout.size()<<"\n";
    }

    unsigned totalNumBlks = _curLayout->size() + _placedBlocks.size();
    unsigned minBlkSize = _hgraphOfNetlist.getNumNodes()/totalNumBlks;
    minBlkSize = minBlkSize < 50 ? 0 : minBlkSize/2;
	//min becomes 0 when we get to small block partitioning
	//so that we can't get stuck, unable to partition a very small
	//multi-row block

    cerr<<"   [";

    for(unsigned b = 0; b < curLayout.size(); b++)
    {
	curLayout[b]->_canSplitBlock = true;
	curLayout[b]->_isNewBlock    = false;
    }

    if(_params.verb.forMajStats > 1)
	    cout<<"H-Split pass"<<endl;

    vector<CapoBlock*> tempNewLayout;
    tempNewLayout.reserve(curLayout.size()*2);

	//now, do horizontal splits
    for(b = 0; b < curLayout.size(); b++)
    {
	CapoBlock& block = *curLayout[b];
	_totalOverfill += block.getOverfill();

	if(!block.canSplitBlock())
	{
	   if(_params.verb.forActions > 5)
		cout<<"Already partitioned: blk "<<b<<"("
			<<block.getIndex()<<")"<<endl;
	   //will all be deleted after this loop
	   continue;
	}

	if(_params.verb.forMajStats > 2)
	{
	       cout<<"Block: "<<b<<" ["<<block.getNumCells() <<" cells]"<<endl;
	       if(_params.verb.forMajStats > 4)
	           cout<<block<<endl;
	}

	refineBlock(block,tempNewLayout,_layerNum, HOnly, minBlkSize);
    }

    cerr<<"]";

	//blocks may have been passed over initially, then
	//split as a member of a chain..in which case they woul
	//already have been transfered to 'tempNewLayout'..
    for(b = 0; b < tempNewLayout.size(); b++)
    {
	if(tempNewLayout[b]->isNewBlock() ||
	   tempNewLayout[b]->canSplitBlock())
		newLayout.push_back(tempNewLayout[b]);
    }

    for(b = 0; b < curLayout.size(); b++)
    {
	if(!curLayout[b]->canSplitBlock())	//was split....delete it
	    delete curLayout[b];

	curLayout[b] = 0;
    }
    curLayout.clear();

}

void CapoPlacer::doHAndVLayer(vector<CapoBlock*>& curLayout,
			      vector<CapoBlock*>& newLayout,
			      unsigned layerNum)
{
    if(_params.verb.forMajStats > 0)
    {
        cerr<<"Layer:    "<<layerNum<<"  Blocks: "<<curLayout.size()<<"\n";
        cout<<"Layer:    "<<layerNum<<"  Blocks: "<<curLayout.size()<<"\n";
    }

     unsigned minBlkSize = 0; //will partition everything

    for(unsigned b = 0; b < curLayout.size(); b++)
	    curLayout[b]->_canSplitBlock = true;

    cerr<<"   [";

    for(b = 0; b < curLayout.size(); b++)
    {
	CapoBlock& block = *curLayout[b];
	_totalOverfill += block.getOverfill();

	if(_params.verb.forMajStats > 2)
	{
	       cout<<"Block: "<<b<<" ["<<block.getNumCells() <<" cells]"<<endl;
	       if(_params.verb.forMajStats > 4)
	           cout<<block<<endl;
	}

	bool splitBlock = refineBlock(block,newLayout,_layerNum, 
						HAndV,minBlkSize);
	if(splitBlock)
	    delete curLayout[b];
	curLayout[b] = 0;
    }
    cerr<<"]";

    curLayout.clear();
}

